"""PTR Command Line Interface module."""
import os
import shutil as sh
from argparse import ArgumentParser

from .simphony import SimphonyWrapper
from .simphony import create_structure
from .simphony import remove_directory_if_empty


def cli(test=False):
    """CLI Resolve a PTR and generate a SPICE CK."""
    parser = ArgumentParser(description='Simulate a PTR and generate the corresponding SPICE CK, resolved PTR, '
                                        'available Power, and quaternions dump file.')

    # Exclusive group for session-file and the other parameters
    exclusive_group = parser.add_mutually_exclusive_group(required=False)
    exclusive_group.add_argument("-s", "--session-file", help="Path to the session file. "
                                 "If provided the other arguments are ignored.")

    parser.add_argument("-m", "--meta-kernel", help="Path to the meta-kernel file.")
    parser.add_argument("-p", "--ptr", help="Path to the PTR/PTX file with")
    parser.add_argument("-w", "--working-dir", default=os.getcwd(),
                        help="Path to the working directory. Default is the current directory.")
    parser.add_argument("-o", "--output-dir", help="Path to the output directory. Overwrites "
                        "default output file names. Default is the structure of the built-in session file.")
    args = parser.parse_args()

    # Process the arguments and perform further actions
    if args.session_file:
        # Handle session-file exclusively
        # Do something with args.session_file
        session_file_path = args.session_file
        if not os.path.exists(session_file_path):
            raise ValueError("session-file does not exist")

        session_file_path = os.path.abspath(session_file_path)
        root_scenario_path = os.path.dirname(os.path.abspath(session_file_path))
    else:
        # Handle the other parameters exclusively
        if args.ptr is None:
            parser.error("PTR argument is missing")

        if args.meta_kernel is None:
            parser.error("META_KERNEL argument is missing")

        try:
            with open(args.ptr, 'r') as p:
                ptr_content = p.read()
        except FileNotFoundError:
            raise ValueError("PTR/PTX file not found")

        if not os.path.exists(args.meta_kernel):
            raise ValueError("meta-kernel does not exist")

        session_file_path = create_structure(args.working_dir, args.meta_kernel, ptr_content)
        root_scenario_path = os.path.dirname(session_file_path)

    sw = SimphonyWrapper()

    print("ptwrapper session execution")
    print(f"AGM version:  {sw.get_agm_version().decode('UTF-8')}")
    print(f"OSVE version: {sw.get_app_version().decode('UTF-8')}")

    sw.get_attitude(root_scenario_path, session_file_path)

    if args.output_dir:
        # Post-process the result
        fname = args.ptr.split(os.sep)[-1]
        if '.xml' in fname:
            fname = fname.split('.xml')[0]
        elif '.XML' in fname:
            fname = fname.split('.XML')[0]
        elif '.PTX' in fname:
            fname = fname.split('.PTX')[0]
        elif '.ptx' in fname:
            fname = fname.split('.ptx')[0]
        elif '.ptr' in fname:
            fname = fname.split('.ptr')[0]
        elif '.PTR' in fname:
            fname = fname.split('.PTR')[0]
        else:
            raise ValueError('Input PTR extension incorrect (not .xml, .XML ,.ptr, .PTR, .ptx or .PTX)')

        mkname = args.meta_kernel.split(os.sep)[-1]
        output_dir = os.path.abspath(args.output_dir)

        print(f'Renaming quaternions.csv to {fname}_quaternions.csv')
        sh.move(f'{root_scenario_path}/output/quaternions.csv', f'{output_dir}/{fname}_quaternions.csv')

        print(f'Renaming power.csv to {fname}_power.csv')
        sh.move(f'{root_scenario_path}/output/power.csv', f'{output_dir}/{fname}_power.csv')

        print(f'Renaming ptr_resolved.ptx to {fname}_resolved.ptx')
        sh.move(f'{root_scenario_path}/output/ptr_resolved.ptx', f'{output_dir}/{fname}_resolved.ptx')

        print(f'Renaming juice_sc_ptr.bc to {fname}.bc')
        sh.move(f'{root_scenario_path}/output/juice_sc_ptr.bc', f'{output_dir}/{fname}.bc')

        print(f'Renaming log.json to {fname}_log.json')
        sh.move(f'{root_scenario_path}/output/log.json', f'{output_dir}/{fname}_log.json')

        print('Cleaning up OSVE execution files and directories')
        os.remove(f'{root_scenario_path}/ptr/PTR_PT_V1.ptx')
        os.remove(f'{root_scenario_path}/ptr/downlink.evf')
        os.remove(f'{root_scenario_path}/ptr/TOP_events.evf')
        sh.rmtree(f'{root_scenario_path}/ptr/edf')
        sh.rmtree(f'{root_scenario_path}/ptr/itl')
        os.remove(f'{root_scenario_path}/session_file.json')
        sh.rmtree(f'{root_scenario_path}/config/age')
        sh.rmtree(f'{root_scenario_path}/config/ise')

        if args.meta_kernel != f'{root_scenario_path}/kernel/{mkname}':
            os.remove(f'{root_scenario_path}/kernel/{mkname}')

        for directory in ['kernel', 'output', 'ptr', 'config']:
            remove_directory_if_empty(f'{root_scenario_path}/{directory}')

    if test:
        return parser
